using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Windows;

namespace EditableCollectionSample
{
    public enum Gender
    {
        Female,
        Male,
        Unknown
    }

    public class Character : INotifyPropertyChanged, IEditableObject
    {
        private string _first = string.Empty;
        public string First
        {
            get { return _first; }
            set
            {
                _first = value;
                RaisePropertyChanged("First");
            }
        }

        private string _last = string.Empty;
        public string Last
        {
            get { return _last; }
            set
            {
                _last = value;
                RaisePropertyChanged("Last");
            }
        }

        private string _image = null;
        public string Image
        {
            get { return _image; }
            set
            {
                _image = value;
                RaisePropertyChanged("Image");
            }
        }

        private Gender _gender = Gender.Unknown;
        public Gender Gender
        {
            get { return _gender; }
            set
            {
                _gender = value;
                RaisePropertyChanged("Gender");
            }
        }

        public override string ToString()
        {
            return _first + " " + _last;
        }
        
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        protected void RaisePropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        #endregion

        #region IEditableObject support

        public Character _cachedCopy = null;

        public void BeginEdit()
        {
            // save object state before entering edit mode
            _cachedCopy = new Character();
            _cachedCopy._first = _first;
            _cachedCopy._last = _last;
            _cachedCopy._image = _image;
            _cachedCopy._gender = _gender;

            // ensure edit mode flag is set
            IsInEditMode = true;
        }

        public void CancelEdit()
        {
            // restore original object state
            if (_cachedCopy != null)
            {
                First = _cachedCopy._first;
                Last = _cachedCopy._last;
                Image = _cachedCopy._image;
                Gender = _cachedCopy._gender;
            }

            // clear cached data
            _cachedCopy = null;

            // ensure edit mode flag is unset
            IsInEditMode = false;
        }

        public void EndEdit()
        {
            // clear cached data
            _cachedCopy = null;

            // ensure edit mode flag is unset
            IsInEditMode = false;
        }

        private bool _isInEditMode = false;
        public bool IsInEditMode
        {
            get { return _isInEditMode; }
            private set
            {
                if (_isInEditMode != value)
                {
                    _isInEditMode = value;
                    RaisePropertyChanged("IsInEditMode");
                }
            }
        }

        #endregion
    }
}
