using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Xml;
using System.Threading;
using System.Windows.Threading;
using System.Windows.Controls.Primitives;

namespace EditableCollectionSample
{
    public partial class Window1 : System.Windows.Window
    {
        public Window1()
        {
            InitializeComponent();

            // once the element tree is loaded, focus the first listbox item
            Dispatcher.BeginInvoke(DispatcherPriority.Loaded, (ThreadStart)delegate()
            {
                IInputElement container = lb.ItemContainerGenerator.ContainerFromIndex(0) as IInputElement;
                if (container != null)
                {
                    container.Focus();
                }
            });
        }

        private void CommitEditOnLostFocus(object sender, DependencyPropertyChangedEventArgs e)
        {
            // if the root element of the edit mode template loses focus, commit the edit
            if ((bool)e.NewValue == false)
            {
                CommitCharacterChanges(null, null);
            }
        }

        private void CharacterDoubleClick(object sender, RoutedEventArgs e)
        {
            EditCharacter(null, null);
        }

        private void EditCharacter(object sender, ExecutedRoutedEventArgs e)
        {
            IEditableCollectionView ecv = lb.Items as IEditableCollectionView;
            Character selectedCharacter = lb.Items.CurrentItem as Character;
            if (selectedCharacter != null && !ecv.IsEditingItem)
            {
                ecv.EditItem(selectedCharacter);

                // invoke focus update at loaded priority so that template swap has time to complete
                Dispatcher.BeginInvoke(DispatcherPriority.Loaded, (ThreadStart)delegate()
                {
                    UIElement container = lb.ItemContainerGenerator.ContainerFromItem(selectedCharacter) as UIElement;
                    if (container != null)
                    {
                        container.MoveFocus(new TraversalRequest(FocusNavigationDirection.Next));
                    }
                });
            }
        }

        private void CommitCharacterChanges(object sender, ExecutedRoutedEventArgs e)
        {
            IEditableCollectionView ecv = lb.Items as IEditableCollectionView;
            Character selectedCharacter = lb.Items.CurrentItem as Character;
            if (selectedCharacter != null && ecv.IsEditingItem && ecv.CurrentEditItem == selectedCharacter)
            {
                ecv.CommitEdit();
                lb.Items.MoveCurrentTo(selectedCharacter);
            }
        }

        private void CancelCharacterChanges(object sender, ExecutedRoutedEventArgs e)
        {
            IEditableCollectionView ecv = lb.Items as IEditableCollectionView;
            Character selectedCharacter = lb.SelectedItem as Character;
            if (selectedCharacter != null && ecv.IsEditingItem && ecv.CurrentEditItem == selectedCharacter)
            {
                ecv.CancelEdit();
                lb.Items.MoveCurrentTo(selectedCharacter);
            }
        }

        private void SelectAllText(object sender, RoutedEventArgs e)
        {
            (sender as TextBox).SelectAll();
        }
    }
}